@@ -1,5 +1,5 @@
 /* Hierarchial argument parsing help output
-   Copyright (C) 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1995,1996,1997,1998,1999,2000 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Written by Miles Bader <miles@gnu.ai.mit.edu>.
 
@@ -45,12 +45,6 @@ char *alloca ();
 # endif
 #endif
 
-#if __GNUC__ && HAVE_GCC_ATTRIBUTE
-# define UNUSED __attribute__ ((__unused__))
-#else
-# define UNUSED
-#endif
-
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -59,55 +53,73 @@ char *alloca ();
 #include <malloc.h>
 #include <ctype.h>
 
+
 #ifndef _
 /* This is for other GNU distributions with internationalized messages.  */
-# ifdef HAVE_LIBINTL_H
+# if defined HAVE_LIBINTL_H || defined _LIBC
 #  include <libintl.h>
+#  ifdef _LIBC
+#   undef dgettext
+#   define dgettext(domain, msgid) __dcgettext (domain, msgid, LC_MESSAGES)
+#  endif
 # else
-#  define dgettext(domain, msgid) (((void) (domain), (msgid)))
+#  define dgettext(domain, msgid) (msgid)
 # endif
 #endif
 
-#ifdef USE_IN_LIBIO
-# define flockfile(s) _IO_flockfile (s)
-# define funlockfile(s) _IO_funlockfile (s)
-#endif
-
 #include "argp.h"
 #include "argp-fmtstream.h"
 #include "argp-namefrob.h"
 
+
+#ifndef UNUSED
+# if __GNUC__ >= 2
+#  define UNUSED __attribute__ ((__unused__))
+# else
+#  define UNUSED
+# endif
+#endif
+
+#ifndef _LIBC
+# ifndef __strchrnul
+#  define __strchrnul strchrnul
+# endif
+# ifndef __mempcpy
+#  define __mempcpy mempcpy
+# endif
+/* We need to use a different name, as __strndup is likely a macro. */
+# define STRNDUP strndup
+# ifndef __flockfile
+#  define __flockfile flockfile
+# endif
+# ifndef __funlockfile
+#  define __funlockfile funlockfile
+# endif
+# if HAVE_STRERROR
+#  define STRERROR strerror
+# else
+#  define STRERROR(x) (sys_errlist[x])
+# endif
+#else /* _LIBC */
+# define STRNDUP __strndup
+# define STRERROR strerror
+#endif
+
 #if !_LIBC
 # if !HAVE_STRNDUP
-
-static char *strndup (const char *s, size_t size)
-{
-  char *r;
-  char *end = memchr(s, 0, size);
-  
-  if (end)
-    /* Length + 1 */
-    size = end - s + 1;
-  
-  r = malloc(size);
-
-  if (size)
-    {
-      memcpy(r, s, size-1);
-      r[size-1] = '\0';
-    }
-  return r;
-}
+char *strndup (const char *s, size_t size);
 # endif /* !HAVE_STRNDUP */
+
 # if !HAVE_MEMPCPY
-static void *mempcpy (void *to, const void *from, size_t size)
-{
-  memcpy(to, from, size);
-  return (char *) to + size;
-}
-# define __mempcpy mempcpy
+void *mempcpy (void *to, const void *from, size_t size);
 # endif /* !HAVE_MEMPCPY */
-#endif /* !_LIBC - 0 */
+
+# if !HAVE_STRCHRNUL
+char *strchrnul(const char *s, int c);
+# endif /* !HAVE_STRCHRNUL */
+
+#endif /* !_LIBC */
+
 
 /* User-selectable (using an environment variable) formatting parameters.
 
@@ -189,9 +201,11 @@ static const struct uparam_name uparam_names[] =
 static void
 fill_in_uparams (const struct argp_state *state)
 {
-  const unsigned char *var = getenv ("ARGP_HELP_FMT");
+  /* FIXME: Can we define var as a pointer to _unsigned_ char,
+   * or will that clash with the getenv() prototype? */
+  const char *var = getenv ("ARGP_HELP_FMT");
 
-#define SKIPWS(p) do { while (isspace (*p)) p++; } while (0);
+#define SKIPWS(p) do { while (isspace ( (unsigned) *p)) p++; } while (0);
 
   if (var)
     /* Parse var. */
@@ -199,14 +213,14 @@ fill_in_uparams (const struct argp_state *state)
       {
 	SKIPWS (var);
 
-	if (isalpha (*var))
+	if (isalpha ( (unsigned) *var))
 	  {
 	    size_t var_len;
 	    const struct uparam_name *un;
 	    int unspec = 0, val = 0;
-	    const unsigned char *arg = var;
+	    const char *arg = var;
 
-	    while (isalnum (*arg) || *arg == '-' || *arg == '_')
+	    while (isalnum ( (unsigned) *arg) || *arg == '-' || *arg == '_')
 	      arg++;
 	    var_len = arg - var;
 
@@ -231,10 +245,10 @@ fill_in_uparams (const struct argp_state *state)
 		else
 		  val = 1;
 	      }
-	    else if (isdigit (*arg))
+	    else if (isdigit ( (unsigned) *arg))
 	      {
 		val = atoi (arg);
-		while (isdigit (*arg))
+		while (isdigit ( (unsigned) *arg))
 		  arg++;
 		SKIPWS (arg);
 	      }
@@ -400,7 +414,7 @@ struct hol_cluster
   const char *header;
 
   /* Used to order clusters within the same group with the same parent,
-     according to the order in which they occured in the parent argp's child
+     according to the order in which they occurred in the parent argp's child
      list.  */
   int index;
 
@@ -728,12 +742,12 @@ canon_doc_option (const char **name)
 {
   int non_opt;
   /* Skip initial whitespace.  */
-  while (isspace ( (unsigned char) **name))
+  while (isspace ( (unsigned) **name))
     (*name)++;
   /* Decide whether this looks like an option (leading `-') or not.  */
   non_opt = (**name != '-');
   /* Skip until part of name used for sorting.  */
-  while (**name && !isalnum ( (unsigned char) **name))
+  while (**name && !isalnum ( (unsigned) **name))
     (*name)++;
   return non_opt;
 }
@@ -795,12 +809,12 @@ hol_entry_cmp (const struct hol_entry *entry1,
 	   first, but as they're not displayed, it doesn't matter where
 	   they are.  */
 	{
-	  unsigned char first1 = short1 ? short1 : long1 ? *long1 : 0;
-	  unsigned char first2 = short2 ? short2 : long2 ? *long2 : 0;
+	  char first1 = short1 ? short1 : long1 ? *long1 : 0;
+	  char first2 = short2 ? short2 : long2 ? *long2 : 0;
 #ifdef _tolower
-	  int lower_cmp = _tolower (first1) - _tolower (first2);
+	  int lower_cmp = _tolower ( (unsigned) first1) - _tolower ( (unsigned) first2);
 #else
-	  int lower_cmp = tolower (first1) - tolower (first2);
+	  int lower_cmp = tolower ( (unsigned) first1) - tolower ( (unsigned) first2);
 #endif
 	  /* Compare ignoring case, except when the options are both the
 	     same letter, in which case lower-case always comes first.  */
@@ -946,7 +960,7 @@ space (argp_fmtstream_t stream, size_t ensure)
    optional argument.  */
 static void
 arg (const struct argp_option *real, const char *req_fmt, const char *opt_fmt,
-     const char *domain, argp_fmtstream_t stream)
+     const char *domain UNUSED, argp_fmtstream_t stream)
 {
   if (real->arg)
     {
@@ -1093,18 +1107,8 @@ hol_entry_help (struct hol_entry *entry, const struct argp_state *state,
   int old_wm = __argp_fmtstream_wmargin (stream);
   /* PEST is a state block holding some of our variables that we'd like to
      share with helper functions.  */
-#ifdef __GNUC__
   struct pentry_state pest = { entry, stream, hhstate, 1, state };
-#else /* !__GNUC__ */
-  /* Decent initializers is a GNU extension */
-  struct pentry_state pest;
-  pest.entry = entry;
-  pest.stream = stream;
-  pest.hhstate = hhstate;
-  pest.first = 1;
-  pest.state = state;
-#endif /* !__GNUC__ */
-  
+
   if (! odoc (real))
     for (opt = real, num = entry->num; num > 0; opt++, num--)
       if (opt->name && ovisible (opt))
@@ -1268,7 +1272,7 @@ add_argless_short_opt (const struct argp_option *opt,
 static int
 usage_argful_short_opt (const struct argp_option *opt,
 			const struct argp_option *real,
-			const char *domain, void *cookie)
+			const char *domain UNUSED, void *cookie)
 {
   argp_fmtstream_t stream = cookie;
   const char *arg = opt->arg;
@@ -1424,19 +1428,17 @@ argp_args_usage (const struct argp *argp, const struct argp_state *state,
   if (fdoc)
     {
       const char *cp = fdoc;
-      nl = strchr (cp, '\n');
-      if (nl)
+      nl = __strchrnul (cp, '\n');
+      if (*nl != '\0')
 	/* This is a `multi-level' args doc; advance to the correct position
 	   as determined by our state in LEVELS, and update LEVELS.  */
 	{
 	  int i;
 	  multiple = 1;
 	  for (i = 0; i < *our_level; i++)
-	    cp = nl + 1, nl = strchr (cp, '\n');
+	    cp = nl + 1, nl = __strchrnul (cp, '\n');
 	  (*levels)++;
 	}
-      if (! nl)
-	nl = cp + strlen (cp);
 
       /* Manually do line wrapping so that it (probably) won't get wrapped at
 	 any embedded spaces.  */
@@ -1474,7 +1476,7 @@ argp_args_usage (const struct argp *argp, const struct argp_state *state,
    following the `\v' character (nothing for strings without).  Each separate
    bit of documentation is separated a blank line, and if PRE_BLANK is true,
    then the first is as well.  If FIRST_ONLY is true, only the first
-   occurance is output.  Returns true if anything was output.  */
+   occurrence is output.  Returns true if anything was output.  */
 static int
 argp_doc (const struct argp *argp, const struct argp_state *state,
 	  int post, int pre_blank, int first_only,
@@ -1502,7 +1504,7 @@ argp_doc (const struct argp *argp, const struct argp_state *state,
     {
       if (inp_text_limit)
 	/* Copy INP_TEXT so that it's nul-terminated.  */
-	inp_text = strndup (inp_text, inp_text_limit);
+	inp_text = STRNDUP (inp_text, inp_text_limit);
       input = __argp_input (argp, state);
       text =
 	(*argp->help_filter) (post
@@ -1565,9 +1567,10 @@ argp_doc (const struct argp *argp, const struct argp_state *state,
    argp_state_help, STATE is the relevent parsing state.  FLAGS are from the
    set ARGP_HELP_*.  NAME is what to use wherever a `program name' is
    needed. */
+
 static void
 _help (const struct argp *argp, const struct argp_state *state, FILE *stream,
-       unsigned flags, char *name)
+       unsigned flags, const char *name)
 {
   int anything = 0;		/* Whether we've output anything.  */
   struct hol *hol = 0;
@@ -1576,7 +1579,7 @@ _help (const struct argp *argp, const struct argp_state *state, FILE *stream,
   if (! stream)
     return;
 
-  FLOCKFILE (stream);
+  __flockfile (stream);
 
   if (! uparams.valid)
     fill_in_uparams (state);
@@ -1584,7 +1587,7 @@ _help (const struct argp *argp, const struct argp_state *state, FILE *stream,
   fs = __argp_make_fmtstream (stream, 0, uparams.rmargin, 0);
   if (! fs)
     {
-      FUNLOCKFILE (stream);
+      __funlockfile (stream);
       return;
     }
 
@@ -1692,7 +1695,7 @@ Try `%s --help' or `%s --usage' for more information.\n"),
       anything = 1;
     }
 
-  FUNLOCKFILE (stream);
+  __funlockfile (stream);
 
   if (hol)
     hol_free (hol);
@@ -1711,6 +1714,29 @@ void __argp_help (const struct argp *argp, FILE *stream,
 weak_alias (__argp_help, argp_help)
 #endif
 
+char *__argp_basename(char *name)
+{
+  char *short_name = strrchr(name, '/');
+  return short_name ? short_name + 1 : name;
+}
+
+static const char *
+short_program_name(const struct argp_state *state)
+{
+  if (state)
+    return state->name;
+#if HAVE_PROGRAM_INVOCATION_SHORT_NAME
+  return program_invocation_short_name;
+#elif HAVE_PROGRAM_INVOCATION_NAME
+  return __argp_basename(program_invocation_name);
+#else
+  /* FIXME: What now? Miles suggests that it is better to use NULL,
+     but currently the value is passed on directly to fputs_unlocked,
+     so that requires more changes. */
+  return "";
+#endif
+}
+
 /* Output, if appropriate, a usage message for STATE to STREAM.  FLAGS are
    from the set ARGP_HELP_*.  */
 void
@@ -1722,7 +1748,7 @@ __argp_state_help (const struct argp_state *state, FILE *stream, unsigned flags)
 	flags |= ARGP_HELP_LONG_ONLY;
 
       _help (state ? state->root_argp : 0, state, stream, flags,
-	     state ? state->name : program_invocation_short_name);
+	     short_program_name(state));
 
       if (!state || ! (state->flags & ARGP_NO_EXIT))
 	{
@@ -1751,22 +1777,22 @@ __argp_error (const struct argp_state *state, const char *fmt, ...)
 	{
 	  va_list ap;
 
-	  FLOCKFILE (stream);
+	  __flockfile (stream);
 
-	  FPUTS_UNLOCKED (state ? state->name : program_invocation_short_name,
+	  fputs_unlocked (short_program_name(state),
 			  stream);
-	  PUTC_UNLOCKED (':', stream);
-	  PUTC_UNLOCKED (' ', stream);
+	  putc_unlocked (':', stream);
+	  putc_unlocked (' ', stream);
 
 	  va_start (ap, fmt);
 	  vfprintf (stream, fmt, ap);
 	  va_end (ap);
 
-	  PUTC_UNLOCKED ('\n', stream);
+	  putc_unlocked ('\n', stream);
 
 	  __argp_state_help (state, stream, ARGP_HELP_STD_ERR);
 
-	  FUNLOCKFILE (stream);
+	  __funlockfile (stream);
 	}
     }
 }
@@ -1792,17 +1818,17 @@ __argp_failure (const struct argp_state *state, int status, int errnum,
 
       if (stream)
 	{
-	  FLOCKFILE (stream);
+	  __flockfile (stream);
 
-	  FPUTS_UNLOCKED (state ? state->name : program_invocation_short_name,
+	  fputs_unlocked (short_program_name(state),
 			  stream);
 
 	  if (fmt)
 	    {
 	      va_list ap;
 
-	      PUTC_UNLOCKED (':', stream);
-	      PUTC_UNLOCKED (' ', stream);
+	      putc_unlocked (':', stream);
+	      putc_unlocked (' ', stream);
 
 	      va_start (ap, fmt);
 	      vfprintf (stream, fmt, ap);
@@ -1811,14 +1837,14 @@ __argp_failure (const struct argp_state *state, int status, int errnum,
 
 	  if (errnum)
 	    {
-	      PUTC_UNLOCKED (':', stream);
-	      PUTC_UNLOCKED (' ', stream);
-	      fputs (strerror (errnum), stream);
+	      putc_unlocked (':', stream);
+	      putc_unlocked (' ', stream);
+	      fputs (STRERROR (errnum), stream);
 	    }
 
-	  PUTC_UNLOCKED ('\n', stream);
+	  putc_unlocked ('\n', stream);
 
-	  FUNLOCKFILE (stream);
+	  __funlockfile (stream);
 
 	  if (status && (!state || !(state->flags & ARGP_NO_EXIT)))
 	    exit (status);
