@@ -1,5 +1,5 @@
-/* Hierarchial argument parsing, layered over getopt.
-   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
+/* Hierarchial argument parsing.
+   Copyright (C) 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Written by Miles Bader <miles@gnu.ai.mit.edu>.
 
@@ -24,41 +24,13 @@
 #include <stdio.h>
 #include <ctype.h>
 
-#if LSH
-# if !PRINTF_STYLE
-#  if HAVE_GCC_ATTRIBUTE
-#   define PRINTF_STYLE(f, a) __attribute__ ((__format__ (__printf__, f, a)))
-#  else
-#   define PRINTF_STYLE(f, a)
-#  endif /* HAVE_GCC_ATTRIBUTE */
-# endif /* !PRINTF_STYLE */
-# if HAVE_GETOPT_LONG
-#  include <getopt.h>
-# else
-#  include "getopt.h"
-# endif /* !HAVE_GETOPT_LONG */
-# ifndef __THROW
-#  define __THROW
-# endif
-/* FIXME: We should really use a configure test for __restrict */
-# ifndef __restrict
-#  define __restrict
-# endif
-#else /* !LSH */
-# include <getopt.h>
-#endif /* !LSH */
-
-#if !HAVE_PROGRAM_INVOCATION_NAME
-extern char *program_invocation_name;
-#endif
-
-#if !HAVE_PROGRAM_INVOCATION_SHORT_NAME
-extern char *program_invocation_short_name;
-#endif
-
 #define __need_error_t
 #include <errno.h>
 
+#ifndef __THROW
+# define __THROW
+#endif
+
 #ifndef __const
 # define __const const
 #endif
@@ -67,27 +39,6 @@ extern char *program_invocation_short_name;
 typedef int error_t;
 # define __error_t_defined
 #endif
-
-#ifndef __P
-# if (defined __STDC__ && __STDC__) || defined __cplusplus
-#  if defined __GNUC__ && defined __cplusplus && __GNUC_MINOR__ >= 8
-#   define __P(args)	args throw ()
-#  else
-#   define __P(args)	args
-#  endif
-# else
-#  define __P(args)	()
-# endif
-#endif
-
-#ifndef __PMT
-# if (defined __STDC__ && __STDC__) || defined __cplusplus
-#  define __PMT(args)	args
-# else
-#  define __PMT(args)	()
-# endif
-#endif
-
 
 #ifdef  __cplusplus
 extern "C" {
@@ -168,8 +119,8 @@ struct argp_state;		/* " */
 struct argp_child;		/* " */
 
 /* The type of a pointer to an argp parsing function.  */
-typedef error_t (*argp_parser_t) __PMT ((int key, char *arg,
-					 struct argp_state *state));
+typedef error_t (*argp_parser_t) (int key, char *arg,
+				  struct argp_state *state);
 
 /* What to return for unrecognized keys.  For special ARGP_KEY_ keys, such
    returns will simply be ignored.  For user keys, this error will be turned
@@ -236,7 +187,7 @@ typedef error_t (*argp_parser_t) __PMT ((int key, char *arg,
    argp's, and perhaps a function to filter help output.  When actually
    parsing options, getopt is called with the union of all the argp
    structures chained together through their CHILD pointers, with conflicts
-   being resolved in favor of the first occurance in the chain.  */
+   being resolved in favor of the first occurrence in the chain.  */
 struct argp
 {
   /* An array of argp_option structures, terminated by an entry with both
@@ -282,8 +233,7 @@ struct argp
      has been done, so if any of the replacement text also needs translation,
      that should be done by the filter function.  INPUT is either the input
      supplied to argp_parse, or NULL, if argp_help was called directly.  */
-  char *(*help_filter) __PMT ((int __key, __const char *__text,
-			       void *__input));
+  char *(*help_filter) (int __key, __const char *__text, void *__input);
 
   /* If non-zero the strings used in the argp library are translated using
      the domain described by this string.  Otherwise the currently installed
@@ -426,15 +376,14 @@ struct argp_state
    routine returned a non-zero value, it is returned; otherwise 0 is
    returned.  This function may also call exit unless the ARGP_NO_HELP flag
    is set.  INPUT is a pointer to a value to be passed in to the parser.  */
-extern error_t argp_parse __P ((__const struct argp *__restrict __argp,
-				int __argc, char **__restrict __argv,
-				unsigned __flags, int *__restrict __arg_index,
-				void *__restrict __input));
-extern error_t __argp_parse __P ((__const struct argp *__restrict __argp,
-				  int __argc, char **__restrict __argv,
-				  unsigned __flags,
-				  int *__restrict __arg_index,
-				  void *__restrict __input));
+extern error_t argp_parse (__const struct argp *__restrict __argp,
+			   int __argc, char **__restrict __argv,
+			   unsigned __flags, int *__restrict __arg_index,
+			   void *__restrict __input) __THROW;
+extern error_t __argp_parse (__const struct argp *__restrict __argp,
+			     int __argc, char **__restrict __argv,
+			     unsigned __flags, int *__restrict __arg_index,
+			     void *__restrict __input) __THROW;
 
 /* Global variables.  */
 
@@ -449,9 +398,9 @@ extern __const char *argp_program_version;
    calls this function with a stream to print the version to and a pointer to
    the current parsing state, and then exits (unless the ARGP_NO_EXIT flag is
    used).  This variable takes precedent over ARGP_PROGRAM_VERSION.  */
-extern void (*argp_program_version_hook) __PMT ((FILE *__restrict __stream,
-						 struct argp_state *__restrict
-						 __state));
+extern void (*argp_program_version_hook) (FILE *__restrict __stream,
+					  struct argp_state *__restrict
+					  __state);
 
 /* If defined or set by the user program, it should point to string that is
    the bug-reporting address for the program.  It will be printed by
@@ -496,12 +445,12 @@ extern error_t argp_err_exit_status;
 
 /* Output a usage message for ARGP to STREAM.  FLAGS are from the set
    ARGP_HELP_*.  */
-extern void argp_help __P ((__const struct argp *__restrict __argp,
-			    FILE *__restrict __stream,
-			    unsigned __flags, char *__restrict __name));
-extern void __argp_help __P ((__const struct argp *__restrict __argp,
-			      FILE *__restrict __stream, unsigned __flags,
-			      char *__name));
+extern void argp_help (__const struct argp *__restrict __argp,
+		       FILE *__restrict __stream,
+		       unsigned __flags, char *__restrict __name) __THROW;
+extern void __argp_help (__const struct argp *__restrict __argp,
+			 FILE *__restrict __stream, unsigned __flags,
+			 char *__name) __THROW;
 
 /* The following routines are intended to be called from within an argp
    parsing routine (thus taking an argp_state structure as the first
@@ -513,28 +462,26 @@ extern void __argp_help __P ((__const struct argp *__restrict __argp,
 
 /* Output, if appropriate, a usage message for STATE to STREAM.  FLAGS are
    from the set ARGP_HELP_*.  */
-extern void argp_state_help __P ((__const struct argp_state *__restrict
-				  __state,
-				  FILE *__restrict __stream,
-				  unsigned int __flags));
-extern void __argp_state_help __P ((__const struct argp_state *__restrict
-				    __state,
-				    FILE *__restrict __stream,
-				    unsigned int __flags));
+extern void argp_state_help (__const struct argp_state *__restrict __state,
+			     FILE *__restrict __stream,
+			     unsigned int __flags) __THROW;
+extern void __argp_state_help (__const struct argp_state *__restrict __state,
+			       FILE *__restrict __stream,
+			       unsigned int __flags) __THROW;
 
 /* Possibly output the standard usage message for ARGP to stderr and exit.  */
-extern void argp_usage __P ((__const struct argp_state *__state));
-extern void __argp_usage __P ((__const struct argp_state *__state));
+extern void argp_usage (__const struct argp_state *__state) __THROW;
+extern void __argp_usage (__const struct argp_state *__state) __THROW;
 
 /* If appropriate, print the printf string FMT and following args, preceded
    by the program name and `:', to stderr, and followed by a `Try ... --help'
    message, then exit (1).  */
-extern void argp_error __P ((__const struct argp_state *__restrict __state,
-			     __const char *__restrict __fmt, ...))
-     PRINTF_STYLE(2,3);
-extern void __argp_error __P ((__const struct argp_state *__restrict __state,
-			       __const char *__restrict __fmt, ...))
-     PRINTF_STYLE(2,3);
+extern void argp_error (__const struct argp_state *__restrict __state,
+			__const char *__restrict __fmt, ...) __THROW
+     __attribute__ ((__format__ (__printf__, 2, 3)));
+extern void __argp_error (__const struct argp_state *__restrict __state,
+			  __const char *__restrict __fmt, ...) __THROW
+     __attribute__ ((__format__ (__printf__, 2, 3)));
 
 /* Similar to the standard gnu error-reporting function error(), but will
    respect the ARGP_NO_EXIT and ARGP_NO_ERRS flags in STATE, and will print
@@ -544,31 +491,37 @@ extern void __argp_error __P ((__const struct argp_state *__restrict __state,
    difference between this function and argp_error is that the latter is for
    *parsing errors*, and the former is for other problems that occur during
    parsing but don't reflect a (syntactic) problem with the input.  */
-extern void argp_failure __P ((__const struct argp_state *__restrict __state,
-			       int __status, int __errnum,
-			       __const char *__restrict __fmt, ...))
-     PRINTF_STYLE(4,5);
-extern void __argp_failure __P ((__const struct argp_state *__restrict __state,
-				 int __status, int __errnum,
-				 __const char *__restrict __fmt, ...))
-     PRINTF_STYLE(4,5);
+extern void argp_failure (__const struct argp_state *__restrict __state,
+			  int __status, int __errnum,
+			  __const char *__restrict __fmt, ...) __THROW
+     __attribute__ ((__format__ (__printf__, 4, 5)));
+extern void __argp_failure (__const struct argp_state *__restrict __state,
+			    int __status, int __errnum,
+			    __const char *__restrict __fmt, ...) __THROW
+     __attribute__ ((__format__ (__printf__, 4, 5)));
 
 /* Returns true if the option OPT is a valid short option.  */
-extern int _option_is_short __P ((__const struct argp_option *__opt));
-extern int __option_is_short __P ((__const struct argp_option *__opt));
+extern int _option_is_short (__const struct argp_option *__opt) __THROW;
+extern int __option_is_short (__const struct argp_option *__opt) __THROW;
 
 /* Returns true if the option OPT is in fact the last (unused) entry in an
    options array.  */
-extern int _option_is_end __P ((__const struct argp_option *__opt));
-extern int __option_is_end __P ((__const struct argp_option *__opt));
+extern int _option_is_end (__const struct argp_option *__opt) __THROW;
+extern int __option_is_end (__const struct argp_option *__opt) __THROW;
 
 /* Return the input field for ARGP in the parser corresponding to STATE; used
    by the help routines.  */
-extern void *_argp_input __P ((__const struct argp *__restrict __argp,
-			       __const struct argp_state *__restrict __state));
-extern void *__argp_input __P ((__const struct argp *__restrict __argp,
-				__const struct argp_state *__restrict
-				__state));
+extern void *_argp_input (__const struct argp *__restrict __argp,
+			  __const struct argp_state *__restrict __state)
+     __THROW;
+extern void *__argp_input (__const struct argp *__restrict __argp,
+			   __const struct argp_state *__restrict __state)
+     __THROW;
+
+/* Used for extracting the program name from argv[0] */
+extern char *_argp_basename(char *name) __THROW;
+extern char *__argp_basename(char *name) __THROW;
+
 
 #ifdef __USE_EXTERN_INLINES
 
