@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.86 2000/11/14 17:00:38 nisse Exp $
+ * $Id: channel.c,v 1.89 2000/11/26 23:25:47 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -168,7 +168,6 @@ prepare_window_adjust(struct ssh_channel *channel,
      (super exception_handler)
      (vars
        (connection object ssh_connection)
-       ;; (table object channel_table)
        ; Non-zero if the channel has already been deallocated.
        (dead . int)
        ; Local channel number 
@@ -279,6 +278,7 @@ make_channel_table(void)
 
   table->global_requests = make_alist(0, -1);
   table->channel_types = make_alist(0, -1);
+  table->open_fallback = NULL;
   
   object_queue_init(&table->local_ports);
   object_queue_init(&table->remote_ports);
@@ -598,12 +598,18 @@ DEFINE_PACKET_HANDLER(static, global_request_handler, connection, packet)
       && parse_atom(&buffer, &name)
       && parse_boolean(&buffer, &want_reply))
     {
-      struct global_request *req;
+      struct global_request *req = NULL;
       struct command_continuation *c = &discard_continuation;
       struct exception_handler *e = connection->e;
-      
-      if (!name || !(req = ALIST_GET(connection->table->global_requests,
-				     name)))
+
+      if (name && connection->table->global_requests)
+	{
+	  CAST_SUBTYPE(global_request, r,
+		       ALIST_GET(connection->table->global_requests,
+				 name));
+	  req = r;
+	}
+      if (!req)
 	{
 	  lsh_string_free(packet);
 
@@ -807,22 +813,37 @@ make_channel_request_exception_handler(struct ssh_connection *connection,
   return &self->super;
 }
 
+static int
+parse_channel_request(struct simple_buffer *buffer,
+		      UINT32 *channel_number,
+		      struct channel_request_info *info)
+{
+  unsigned msg_number;
+
+  if (parse_uint8(buffer, &msg_number)
+      && (msg_number == SSH_MSG_CHANNEL_REQUEST)
+      && parse_uint32(buffer, channel_number)
+      && parse_string(buffer,
+		      &info->type_length, &info->type_data)
+      && parse_boolean(buffer, &info->want_reply))
+    {
+      info->type = lookup_atom(info->type_length, info->type_data);
+      return 1;
+    }
+  else
+    return 0;
+}
+
 DEFINE_PACKET_HANDLER(static, channel_request_handler,
 		      connection, packet)
 {
   struct simple_buffer buffer;
-  unsigned msg_number;
+  struct channel_request_info info;
   UINT32 channel_number;
-  int type;
-  int want_reply;
   
   simple_buffer_init(&buffer, packet->length, packet->data);
 
-  if (parse_uint8(&buffer, &msg_number)
-      && (msg_number == SSH_MSG_CHANNEL_REQUEST)
-      && parse_uint32(&buffer, &channel_number)
-      && parse_atom(&buffer, &type)
-      && parse_boolean(&buffer, &want_reply))
+  if (parse_channel_request(&buffer, &channel_number, &info))
     {
       struct ssh_channel *channel = lookup_channel(connection->table,
 						   channel_number);
@@ -833,14 +854,22 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
 
       if (channel)
 	{
-	  struct channel_request *req;
+	  struct channel_request *req = NULL;
 	  struct command_continuation *c = &discard_continuation;
 	  struct exception_handler *e = channel->e;
 
-	  if (type && channel->request_types 
-	      && ( (req = ALIST_GET(channel->request_types, type)) ))
+	  if (info.type && channel->request_types)
 	    {
-	      if (want_reply)
+	      CAST_SUBTYPE(channel_request, r,
+			   ALIST_GET(channel->request_types, info.type));
+	      req = r;
+	    }
+	  if (!req)
+	    req = channel->request_fallback;
+	  
+	  if (req)
+	    {
+	      if (info.want_reply)
 		{
 		  struct request_status *a = make_request_status();
 		  
@@ -849,7 +878,6 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
 		  
 		  c = make_channel_request_response(connection, channel, a);
 		  e = make_channel_request_exception_handler(connection, channel, a, e, HANDLER_CONTEXT);
-		  
 		}
 	      else
 		{
@@ -862,11 +890,11 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
 						    e, HANDLER_CONTEXT);
 		}
 	      
-	      CHANNEL_REQUEST(req, channel, connection, type, want_reply, &buffer, c, e);
+	      CHANNEL_REQUEST(req, channel, connection, &info, &buffer, c, e);
 	    }
 	  else
 	    {
-	      if (want_reply)
+	      if (info.want_reply)
 		C_WRITE(connection,
 			format_channel_failure(channel->channel_number));
 	    }
@@ -892,8 +920,8 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
        (connection object ssh_connection)
        (local_channel_number . UINT32)
        (remote_channel_number . UINT32)
-       (window_size . UINT32)
-       (max_packet . UINT32)))
+       (send_window_size . UINT32)
+       (send_max_packet . UINT32)))
 */
 
 static void
@@ -908,8 +936,8 @@ do_channel_open_continue(struct command_continuation *c,
   /* FIXME: This copying could just as well be done by the
    * CHANNEL_OPEN handler? Then we can remove the corresponding fields
    * from the closure as well. */
-  channel->send_window_size = self->window_size;
-  channel->send_max_packet = self->max_packet;
+  channel->send_window_size = self->send_window_size;
+  channel->send_max_packet = self->send_max_packet;
   channel->channel_number = self->remote_channel_number;
 
   /* FIXME: Is the channel->write field really needed? */
@@ -930,8 +958,8 @@ static struct command_continuation *
 make_channel_open_continuation(struct ssh_connection *connection,
 			       UINT32 local_channel_number,
 			       UINT32 remote_channel_number,
-			       UINT32 window_size,
-			       UINT32 max_packet)
+			       UINT32 send_window_size,
+			       UINT32 send_max_packet)
 {
   NEW(channel_open_continuation, self);
 
@@ -939,8 +967,8 @@ make_channel_open_continuation(struct ssh_connection *connection,
   self->connection = connection;
   self->local_channel_number = local_channel_number;
   self->remote_channel_number = remote_channel_number;
-  self->window_size = window_size;
-  self->max_packet = max_packet;
+  self->send_window_size = send_window_size;
+  self->send_max_packet = send_max_packet;
 
   return &self->super;
 }
@@ -1002,26 +1030,52 @@ make_exc_channel_open_handler(struct ssh_connection *connection,
   return &self->super;
 }
 
+static int
+parse_channel_open(struct simple_buffer *buffer,
+		   struct channel_open_info *info)
+{
+  unsigned msg_number;
+
+  if (parse_uint8(buffer, &msg_number)
+      && (msg_number == SSH_MSG_CHANNEL_OPEN)
+      && parse_string(buffer, &info->type_length, &info->type_data)
+      && parse_uint32(buffer, &info->remote_channel_number)
+      && parse_uint32(buffer, &info->send_window_size)
+      && parse_uint32(buffer, &info->send_max_packet))
+    {
+      info->type = lookup_atom(info->type_length, info->type_data);
+
+      /* We don't support larger packets than the default,
+       * SSH_MAX_PACKET. The fuzz factor is because the
+       * channel's max sizes refer to the data string inside the
+       * packet, while the SSH_PACKET limit refers to the complete
+       * packet including some overhead (9 octets for
+       * SSH_MSG_CHANNEL_DATA and 13 octets for
+       * SSH_MSG_CHANNEL_EXTENDED_DATA). */
+      if (info->send_max_packet > (SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ))
+	{
+	  werror("do_channel_open: The remote end asked for really large packets.\n");
+	  info->send_max_packet = SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ;
+	}
+
+      return 1;
+    }
+  else
+    return 0;
+}
+
+
 DEFINE_PACKET_HANDLER(static, channel_open_handler,
 		      connection, packet)
 {
   struct simple_buffer buffer;
-  unsigned msg_number;
-  int type;
-  UINT32 remote_channel_number;
-  UINT32 window_size;
-  UINT32 max_packet;
+  struct channel_open_info info;
   
   simple_buffer_init(&buffer, packet->length, packet->data);
 
-  if (parse_uint8(&buffer, &msg_number)
-      && (msg_number == SSH_MSG_CHANNEL_OPEN)
-      && parse_atom(&buffer, &type)
-      && parse_uint32(&buffer, &remote_channel_number)
-      && parse_uint32(&buffer, &window_size)
-      && parse_uint32(&buffer, &max_packet))
+  if (parse_channel_open(&buffer, &info))
     {
-      struct channel_open *open;
+      struct channel_open *open = NULL;
 
       /* NOTE: We can't free the packet yet, as the buffer is passed
        * to the CHANNEL_OPEN method later. */
@@ -1031,56 +1085,55 @@ DEFINE_PACKET_HANDLER(static, channel_open_handler,
 	  /* We are waiting for channels to close. Don't open any new ones. */
 
 	  C_WRITE(connection,
-		  format_open_failure(remote_channel_number,
+		  format_open_failure(info.remote_channel_number,
 				      SSH_OPEN_ADMINISTRATIVELY_PROHIBITED,
 				      "Waiting for channels to close.", ""));
 	}
-      else if (!type || !(open = ALIST_GET(connection->table->channel_types,
-					   type)))
-	{
-	  C_WRITE(connection,
-		  format_open_failure(remote_channel_number,
-				      SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
-				      "Unknown channel type", ""));
-	}
       else
 	{
-      	  int local_number = alloc_channel(connection->table);
-
-	  if (local_number < 0)
-	    C_WRITE(connection,
-		    format_open_failure(remote_channel_number,
-					SSH_OPEN_RESOURCE_SHORTAGE,
-					"Unknown channel type", ""));
-
-	  /* We don't support larger packets than the default,
-	   * SSH_MAX_PACKET. The fuzz factor is because the
-	   * channel's max sizes refer to the data string inside the
-	   * packet, while the SSH_PACKET limit refers to the complete
-	   * packet including some overhead (9 octets for
-	   * SSH_MSG_CHANNEL_DATA and 13 octets for
-	   * SSH_MSG_CHANNEL_EXTENDED_DATA). */
-	  if (max_packet > (SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ))
+	  if (info.type)
 	    {
-	      werror("do_channel_open: The remote end asked for really large packets.\n");
-	      max_packet = SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ;
+	      CAST_SUBTYPE(channel_open, o,
+			   ALIST_GET(connection->table->channel_types,
+				     info.type));
+	      open = o;
+	    }
+
+	  if (!open)
+	    open = connection->table->open_fallback;
+	  
+	  if (!open)
+	    {
+	      C_WRITE(connection,
+		      format_open_failure(info.remote_channel_number,
+					  SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
+					  "Unknown channel type", ""));
+	    }
+	  else
+	    {
+	      int local_number = alloc_channel(connection->table);
+
+	      if (local_number < 0)
+		C_WRITE(connection,
+			format_open_failure(info.remote_channel_number,
+					    SSH_OPEN_RESOURCE_SHORTAGE,
+					    "Channel limit exceeded.", ""));
+
+	      CHANNEL_OPEN(open, connection,
+			   &info,
+			   &buffer,
+			   make_channel_open_continuation(connection,
+							  local_number,
+							  info.remote_channel_number,
+							  info.send_window_size,
+							  info.send_max_packet),
+			   make_exc_channel_open_handler(connection,
+							 local_number,
+							 info.remote_channel_number,
+							 connection->e,
+							 HANDLER_CONTEXT));
+
 	    }
-	    
-	  CHANNEL_OPEN(open, connection,
-		       type,
-		       window_size,
-		       max_packet,
-		       &buffer,
-		       make_channel_open_continuation(connection,
-						      local_number,
-						      remote_channel_number,
-						      window_size,
-						      max_packet),
-		       make_exc_channel_open_handler(connection,
-						     local_number,
-						     remote_channel_number,
-						     connection->e,
-						     HANDLER_CONTEXT));
 	}
     }
   else
@@ -1729,6 +1782,8 @@ init_channel(struct ssh_channel *channel)
   channel->sources = 0;
   
   channel->request_types = NULL;
+  channel->request_fallback = NULL;
+  
   channel->receive = NULL;
   channel->send_adjust = NULL;
 
@@ -1985,6 +2040,19 @@ make_channel_io_exception_handler(struct ssh_channel *channel,
   return &self->super;
 }
 
+/* Used by do_gateway_channel_open() */
+struct lsh_string *
+format_channel_open_s(struct lsh_string *type,
+		      UINT32 local_channel_number,
+		      struct ssh_channel *channel,
+		      struct lsh_string *args)
+{
+  return ssh_format("%c%S%i%i%i%lS", SSH_MSG_CHANNEL_OPEN,
+		    type, local_channel_number, 
+		    channel->rec_window_size, channel->rec_max_packet,
+ 		    args);
+}
+
 struct lsh_string *
 format_channel_open(int type, UINT32 local_channel_number,
 		    struct ssh_channel *channel,
@@ -2021,7 +2089,19 @@ format_channel_open(int type, UINT32 local_channel_number,
 #undef OPEN_FORMAT
 #undef OPEN_ARGS
 }
-		   
+
+struct lsh_string *
+format_channel_request_i(struct channel_request_info *info,
+			 struct ssh_channel *channel,
+			 UINT32 args_length, const UINT8 *args_data)
+{
+  return ssh_format("%c%i%s%c%ls", SSH_MSG_CHANNEL_REQUEST,
+		    channel->channel_number,
+		    info->type_length, info->type_data,
+		    info->want_reply,
+		    args_length, args_data);
+}
+
 struct lsh_string *
 format_channel_request(int type, struct ssh_channel *channel,
 		       int want_reply, const char *format, 
