@@ -1,6 +1,6 @@
 /* disconnect.h
  *
- * $Id: disconnect.h,v 1.5 1999/02/25 00:33:33 nisse Exp $ */
+ * $Id: disconnect.h,v 1.7 2000/04/21 22:29:36 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -28,6 +28,7 @@
 
 struct lsh_string *format_disconnect(int code, const char *msg, 
                                      const char *language);
-struct packet_handler *make_disconnect_handler(void);
+
+extern struct packet_handler disconnect_handler;
 
 #endif /* LSH_DISCONNECT_H_INCLUDED */
