@@ -1,6 +1,6 @@
 /* dsa.c
  *
- * $Id: dsa.c,v 1.22 2000/10/06 11:11:05 nisse Exp $
+ * $Id: dsa.c,v 1.24 2000/10/17 23:48:43 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -37,12 +37,16 @@
 
 #include <assert.h>
 
-#define GABA_DEFINE
-#include "dsa.h.x"
-#undef GABA_DEFINE
-
 #include "dsa.c.x" 
 
+/* The standard says that DSA public keys are at most 1024 bits, i.e.
+ * 128 octets. We are a little more liberal than that. Note that
+ * allowing really large keys opens for Denial-of-service attacks. */
+
+#define DSA_MAX_SIZE 300
+
+#define DSA_MAX_QSIZE SHA_DIGESTSIZE
+
 #define SA(x) sexp_a(ATOM_##x)
 
 /* DSA signatures */
@@ -55,7 +59,32 @@
        (random object randomness)))
 */
 
-static void dsa_hash(mpz_t h, UINT32 length, const UINT8 *msg)
+/* GABA:
+   (class
+     (name dsa_verifier)
+     (super verifier)
+     (vars
+       (p bignum)
+       (q bignum)
+       (g bignum)
+       (y bignum)))
+*/
+
+/* GABA:
+   (class
+     (name dsa_signer)
+     (super signer)
+     (vars
+       (verifier object dsa_verifier)
+       (random object randomness)
+       (a bignum)))
+*/
+
+
+/* Utility functions */
+
+static void
+dsa_hash(mpz_t h, UINT32 length, const UINT8 *msg)
 {
   /* Compute hash */
   struct hash_instance *hash = MAKE_HASH(&sha1_algorithm);
@@ -70,8 +99,6 @@ static void dsa_hash(mpz_t h, UINT32 length, const UINT8 *msg)
   KILL(hash);
 }
 
-#define DSA_MAX_QSIZE SHA_DIGESTSIZE
-
 static struct sexp *
 encode_dsa_sig_val(mpz_t r, mpz_t s)
 {
@@ -93,221 +120,13 @@ decode_dsa_sig_val(struct sexp *e, mpz_t r, mpz_t s)
     }
 }
 
-static void generic_dsa_sign(struct dsa_signer *closure,
-			     UINT32 length, const UINT8 *msg,
-			     mpz_t r, mpz_t s)
-{
-  mpz_t k, tmp;
-
-  assert(r && s);
-  
-  /* Select k, 0<k<q, randomly */
-  mpz_init_set(tmp, closure->public.q);
-  mpz_sub_ui(tmp, tmp, 1);
-
-  mpz_init(k);
-  bignum_random(k, closure->random, tmp);
-  mpz_add_ui(k, k, 1);
-
-  debug("generic_dsa_sign, k: %xn\n", k);
-  
-  /* Compute r = (g^k (mod p)) (mod q) */
-  mpz_powm(r, closure->public.g, k, closure->public.p);
-
-  debug("generic_dsa_sign, group element: %xn\n", r);
-  
-  mpz_fdiv_r(r, r, closure->public.q);
-
-  debug("generic_dsa_sign, r: %xn\n", r);
-
-  /* Compute hash */
-  dsa_hash(tmp, length, msg);
-  
-  /* Compute k^-1 (mod q) */
-  if (!mpz_invert(k, k, closure->public.q))
-    {
-      fatal("generic_dsa_sign: k non-invertible\n");
-    }
-
-  /* Compute signature s = k^-1(h + ar) (mod q) */
-  mpz_mul(s, r, closure->a);
-  mpz_fdiv_r(s, s, closure->public.q);
-  mpz_add(s, s, tmp);
-  mpz_mul(s, s, k);
-  mpz_fdiv_r(s, s, closure->public.q);
-
-  debug("generic_dsa_sign, s: %xn\n", s);
-  
-  mpz_clear(k);
-  mpz_clear(tmp);
-}
-
-static UINT32
-dsa_blob_length(mpz_t r, mpz_t s)
-{
-  UINT32 r_length = bignum_format_u_length(r);
-  UINT32 s_length = bignum_format_u_length(s);
-
-  return MAX(r_length, s_length);
-}
-
-static void
-dsa_blob_write(mpz_t r, mpz_t s, UINT32 length, UINT8 *buf)
-{
-  bignum_write(r, length, buf);
-  bignum_write(s, length, buf + length);
-}
-
-static struct lsh_string *
-do_dsa_sign(struct signer *c,
-	    int algorithm,
-	    UINT32 msg_length,
-	    const UINT8 *msg)
-{
-  CAST(dsa_signer, self, c);
-  mpz_t r, s;
-  struct lsh_string *signature;
-  UINT32 buf_length;
-  UINT8 *p;
-
-  trace("do_dsa_sign: Signing according to %a\n", algorithm);
-  
-  mpz_init(r); mpz_init(s);
-  generic_dsa_sign(self, msg_length, msg, r, s);
-
-  /* Build signature */
-  buf_length = dsa_blob_length(r, s);
-
-  switch (algorithm)
-    {
-    case ATOM_SSH_DSS:
-      /* NOTE: draft-ietf-secsh-transport-X.txt (x <= 07) uses an extra
-       * length field, which should be removed in the next version. */
-      signature = ssh_format("%a%r", ATOM_SSH_DSS, buf_length * 2, &p);
-      dsa_blob_write(r, s, buf_length, p);
-
-      break;
-      
-#if DATAFELLOWS_WORKAROUNDS
-    case ATOM_SSH_DSS_KLUDGE_LOCAL:
-      
-      /* NOTE: This doesn't include any length field. Is that right? */
-      signature = ssh_format("%lr", buf_length * 2, &p);
-      dsa_blob_write(r, s, buf_length, p);
-
-      break;
-
-#endif
-    case ATOM_SPKI:
-      /* NOTE: Generates the <sig-val> only. */
-      signature
-	= sexp_format(encode_dsa_sig_val(r, s),
-		      SEXP_CANONICAL, 0);
-      
-      break;
-    default:
-      fatal("do_dsa_sign: Internal error.");
-    }
-  mpz_clear(r);
-  mpz_clear(s);
-
-  return signature;
-}
-
-static struct sexp *
-do_dsa_sign_spki(struct signer *c,
-		 /* struct sexp *hash, struct sexp *principal, */
-		 UINT32 msg_length,
-		 const UINT8 *msg)
-{
-  CAST(dsa_signer, self, c);
-  mpz_t r, s;
-  struct sexp *signature;
-    
-  mpz_init(r); mpz_init(s);
-  generic_dsa_sign(self, msg_length, msg, r, s);
-      
-  /* Build signature */
-  signature = encode_dsa_sig_val(r, s);
-  
-  mpz_clear(r);
-  mpz_clear(s);
-
-  return signature;
-}
- 
-struct sexp *
-make_dsa_public_key(struct dsa_public *dsa)
-{
-  return sexp_l(5, SA(DSA),
-		sexp_l(2, SA(P), sexp_un(dsa->p), -1),
-		sexp_l(2, SA(Q), sexp_un(dsa->q), -1),
-		sexp_l(2, SA(G), sexp_un(dsa->g), -1),
-		sexp_l(2, SA(Y), sexp_un(dsa->y), -1),
-		-1);
-}
-  
-static struct sexp *
-do_dsa_public_key(struct signer *s)
-{
-  CAST(dsa_signer, self, s);
-
-  return make_dsa_public_key(&self->public);
-}
-
-#if 0
-#if DATAFELLOWS_WORKAROUNDS
-
-static struct lsh_string *
-do_dsa_sign_kludge(struct signer *c,
-		   UINT32 msg_length,
-		   const UINT8 *msg)
-{
-  CAST(dsa_signer_variant, self, c);
-  mpz_t r, s;
-  struct lsh_string *signature;
-  UINT32 buf_length;
-  UINT8 *p;
-
-  mpz_init(r); mpz_init(s);
-  generic_dsa_sign(self->dsa, msg_length, msg, r, s);
-
-  /* Build signature */
-  buf_length = dsa_blob_length(r, s);
-
-  /* NOTE: This doesn't include any length field. Is that right? */
-  signature = ssh_format("%lr", buf_length * 2, &p);
-  dsa_blob_write(r, s, buf_length, p);
-
-  mpz_clear(r);
-  mpz_clear(s);
-
-  return signature;
-}
-
-struct signer *
-make_dsa_signer_kludge(struct signer *s)
-{
-  NEW(dsa_signer_variant, self);
-  CAST(dsa_signer, dsa, s);
-	
-  self->super.sign = do_dsa_sign_kludge;
-  self->super.sign_spki = NULL;
-  
-  self->dsa = dsa;
-  return &self->super;
-}
-#endif
-
-#endif /* DATAFELLOWS_WORKAROUNDS */
-
 
 /* Verifying DSA signatures */
 
 /* The caller should make sure that r and s are non-negative, and not
  * extremely large. That they are less than q is checked here. */
 static int
-generic_dsa_verify(struct dsa_public *key,
+generic_dsa_verify(struct dsa_verifier *key,
 		   UINT32 length,
 		   const UINT8 *msg,
 		   mpz_t r, mpz_t s)
@@ -458,7 +277,7 @@ do_dsa_verify(struct verifier *c, int algorithm,
     default:
       fatal("do_dsa_verify: Internal error!\n");
     }
-  res = generic_dsa_verify(&self->public, length, msg, r, s);
+  res = generic_dsa_verify(self, length, msg, r, s);
  fail:
   mpz_clear(r);
   mpz_clear(s);
@@ -482,8 +301,8 @@ do_dsa_verify_spki(struct verifier *c,
 
   /* NOTE: With the current definition of sexp_get_un, there are no
    * requirements on the order in which r and s occur. */
-  res = decode_dsa_sig_val(e, r, s)
-    && generic_dsa_verify(&self->public, length, msg, r, s);
+  res = (decode_dsa_sig_val(e, r, s)
+	 && generic_dsa_verify(self, length, msg, r, s));
 
   mpz_clear(r);
   mpz_clear(s);
@@ -491,79 +310,258 @@ do_dsa_verify_spki(struct verifier *c,
   return res;
 }
 
-#if 0
-#if DATAFELLOWS_WORKAROUNDS
-
-static int
-do_dsa_verify_kludge(struct verifier *c,
-		     UINT32 length,
-		     const UINT8 *msg,
-		     UINT32 signature_length,
-		     const UINT8 *signature_data)
+static struct lsh_string *
+do_dsa_public_key(struct verifier *s)
 {
-  CAST(dsa_verifier_variant, self, c);
-
-  int res;
-  
-  mpz_t r, s;
-
-  UINT32 buf_length;
-
-  /* NOTE: This doesn't include any length field. Is that right? */
-
-  if ( (signature_length % 2)
-       || (signature_length > (2 * DSA_MAX_QSIZE)) )
-    return 0;
-
-  buf_length = signature_length / 2;
-
-  mpz_init(r);
-  mpz_init(s);
-
-  bignum_parse_u(r, buf_length, signature_data);
-  bignum_parse_u(s, buf_length, signature_data + buf_length);
-    
-  res = generic_dsa_verify(&self->dsa->public, length, msg, r, s);
-  
-  mpz_clear(r);
-  mpz_clear(s);
-
-  return res;
+  CAST(dsa_verifier, self, s);
+  return ssh_format("%a%n%n%n%n",
+		    ATOM_SSH_DSS,
+		    self->p, self->q,
+		    self->g, self->y);
 }
 
+static struct sexp *
+do_dsa_public_spki_key(struct verifier *s)
+{
+  CAST(dsa_verifier, self, s);
+
+  return sexp_l(5, SA(DSA),
+		sexp_l(2, SA(P), sexp_un(self->p), -1),
+		sexp_l(2, SA(Q), sexp_un(self->q), -1),
+		sexp_l(2, SA(G), sexp_un(self->g), -1),
+		sexp_l(2, SA(Y), sexp_un(self->y), -1),
+		-1);
+}
+
+static void
+init_dsa_verifier(struct dsa_verifier *self)
+{
+  /* FIXME: The allocator could do this kind of initialization
+   * automatically. */
+  mpz_init(self->p);
+  mpz_init(self->q);
+  mpz_init(self->g);
+  mpz_init(self->y);
+
+  self->super.verify = do_dsa_verify;
+  self->super.verify_spki = do_dsa_verify_spki;
+  self->super.public_spki_key = do_dsa_public_spki_key;
+  self->super.public_key = do_dsa_public_key;
+}
+
+static struct dsa_verifier *
+make_dsa_verifier_internal(struct sexp_iterator *i)
+{
+  NEW(dsa_verifier, res);
+  init_dsa_verifier(res);
+
+  assert(SEXP_LEFT(i) >= 4);
+
+  if (sexp_get_un(i, ATOM_P, res->p, DSA_MAX_SIZE)
+      && sexp_get_un(i, ATOM_Q, res->q, DSA_MAX_QSIZE)
+      && sexp_get_un(i, ATOM_G, res->g, DSA_MAX_SIZE)
+      && sexp_get_un(i, ATOM_Y, res->y, DSA_MAX_SIZE))
+    {
+      return res;
+    }
+  else
+    {
+      KILL(res);
+      return NULL;
+    }
+}
+  
+/* Alternative constructor using a key of type ssh-dss, when the atom
+ * "ssh-dss" is already read from the buffer. */
 struct verifier *
-make_dsa_verifier_kludge(struct verifier *v)
+parse_ssh_dss_public(struct simple_buffer *buffer)
 {
-  NEW(dsa_verifier_variant, self);
-  CAST(dsa_verifier, dsa, v);
-	
-  self->super.verify = do_dsa_verify_kludge;
+  NEW(dsa_verifier, res);
+  init_dsa_verifier(res);
+
+  if (parse_bignum(buffer, res->p, DSA_MAX_SIZE)
+      && (mpz_sgn(res->p) == 1)
+      && parse_bignum(buffer, res->q, DSA_MAX_QSIZE)
+      && (mpz_sgn(res->q) == 1)
+      && (mpz_cmp(res->q, res->p) < 0) /* q < p */ 
+      && parse_bignum(buffer, res->g, DSA_MAX_SIZE)
+      && (mpz_sgn(res->g) == 1)
+      && (mpz_cmp(res->g, res->p) < 0) /* g < p */ 
+      && parse_bignum(buffer, res->y, DSA_MAX_SIZE) 
+      && (mpz_sgn(res->y) == 1)
+      && (mpz_cmp(res->y, res->p) < 0) /* y < p */
+      && parse_eod(buffer))
+    
+    return &res->super;
+
+  else
+    {
+      KILL(res);
+      return NULL;
+    }
+}
+
   
-  self->dsa = dsa;
-  return &self->super;
+/* Creating signatures */
+static void
+generic_dsa_sign(struct dsa_signer *self,
+		 UINT32 length, const UINT8 *msg,
+		 mpz_t r, mpz_t s)
+{
+  mpz_t k, tmp;
+
+  assert(r && s);
+  
+  /* Select k, 0<k<q, randomly */
+  mpz_init_set(tmp, self->verifier->q);
+  mpz_sub_ui(tmp, tmp, 1);
+
+  mpz_init(k);
+  bignum_random(k, self->random, tmp);
+  mpz_add_ui(k, k, 1);
+
+  debug("generic_dsa_sign, k: %xn\n", k);
+  
+  /* Compute r = (g^k (mod p)) (mod q) */
+  mpz_powm(r, self->verifier->g, k, self->verifier->p);
+
+  debug("generic_dsa_sign, group element: %xn\n", r);
+  
+  mpz_fdiv_r(r, r, self->verifier->q);
+
+  debug("generic_dsa_sign, r: %xn\n", r);
+
+  /* Compute hash */
+  dsa_hash(tmp, length, msg);
+  
+  /* Compute k^-1 (mod q) */
+  if (!mpz_invert(k, k, self->verifier->q))
+    {
+      fatal("generic_dsa_sign: k non-invertible\n");
+    }
+
+  /* Compute signature s = k^-1(h + ar) (mod q) */
+  mpz_mul(s, r, self->a);
+  mpz_fdiv_r(s, s, self->verifier->q);
+  mpz_add(s, s, tmp);
+  mpz_mul(s, s, k);
+  mpz_fdiv_r(s, s, self->verifier->q);
+
+  debug("generic_dsa_sign, s: %xn\n", s);
+  
+  mpz_clear(k);
+  mpz_clear(tmp);
 }
-#endif /* DATAFELLOWS_WORKAROUNDS */
+
+static UINT32
+dsa_blob_length(mpz_t r, mpz_t s)
+{
+  UINT32 r_length = bignum_format_u_length(r);
+  UINT32 s_length = bignum_format_u_length(s);
+
+  return MAX(r_length, s_length);
+}
+
+static void
+dsa_blob_write(mpz_t r, mpz_t s, UINT32 length, UINT8 *buf)
+{
+  bignum_write(r, length, buf);
+  bignum_write(s, length, buf + length);
+}
+
+static struct lsh_string *
+do_dsa_sign(struct signer *c,
+	    int algorithm,
+	    UINT32 msg_length,
+	    const UINT8 *msg)
+{
+  CAST(dsa_signer, self, c);
+  mpz_t r, s;
+  struct lsh_string *signature;
+  UINT32 buf_length;
+  UINT8 *p;
+
+  trace("do_dsa_sign: Signing according to %a\n", algorithm);
+  
+  mpz_init(r); mpz_init(s);
+  generic_dsa_sign(self, msg_length, msg, r, s);
+
+  /* Build signature */
+  buf_length = dsa_blob_length(r, s);
+
+  switch (algorithm)
+    {
+    case ATOM_SSH_DSS:
+      /* NOTE: draft-ietf-secsh-transport-X.txt (x <= 07) uses an extra
+       * length field, which should be removed in the next version. */
+      signature = ssh_format("%a%r", ATOM_SSH_DSS, buf_length * 2, &p);
+      dsa_blob_write(r, s, buf_length, p);
+
+      break;
+      
+#if DATAFELLOWS_WORKAROUNDS
+    case ATOM_SSH_DSS_KLUDGE_LOCAL:
+      
+      /* NOTE: This doesn't include any length field. Is that right? */
+      signature = ssh_format("%lr", buf_length * 2, &p);
+      dsa_blob_write(r, s, buf_length, p);
+
+      break;
+
 #endif
+    case ATOM_SPKI:
+      /* NOTE: Generates the <sig-val> only. */
+      signature
+	= sexp_format(encode_dsa_sig_val(r, s),
+		      SEXP_CANONICAL, 0);
+      
+      break;
+    default:
+      fatal("do_dsa_sign: Internal error.");
+    }
+  mpz_clear(r);
+  mpz_clear(s);
 
-
-/* FIXME: The allocator could do this kind of initialization
- * automatically. */
-void init_dsa_public(struct dsa_public *public)
-{
-  mpz_init(public->p);
-  mpz_init(public->q);
-  mpz_init(public->g);
-  mpz_init(public->y);
+  return signature;
 }
 
-static int
-spki_init_dsa_public(struct dsa_public *key,
-		     struct sexp_iterator *i)
+static struct sexp *
+do_dsa_sign_spki(struct signer *c,
+		 /* struct sexp *hash, struct sexp *principal, */
+		 UINT32 msg_length,
+		 const UINT8 *msg)
 {
-  return (sexp_get_un(i, ATOM_P, key->p, DSA_MAX_SIZE)
-	  && sexp_get_un(i, ATOM_Q, key->q, DSA_MAX_QSIZE)
-	  && sexp_get_un(i, ATOM_G, key->g, DSA_MAX_SIZE)
-	  && sexp_get_un(i, ATOM_Y, key->y, DSA_MAX_SIZE) );
+  CAST(dsa_signer, self, c);
+  mpz_t r, s;
+  struct sexp *signature;
+    
+  mpz_init(r); mpz_init(s);
+  generic_dsa_sign(self, msg_length, msg, r, s);
+      
+  /* Build signature */
+  signature = encode_dsa_sig_val(r, s);
+  
+  mpz_clear(r);
+  mpz_clear(s);
+
+  return signature;
+}
+
+static struct verifier *
+do_dsa_get_verifier(struct signer *s)
+{
+  CAST(dsa_signer, self, s);
+  return &self->verifier->super;
+}
+
+
+static struct verifier *
+make_dsa_verifier(struct signature_algorithm *self UNUSED,
+		  struct sexp_iterator *i)
+{
+  return ( (SEXP_LEFT(i) == 4)
+	   ? &make_dsa_verifier_internal(i)->super
+	   : NULL);
 }
 
 static struct signer *
@@ -572,7 +570,6 @@ make_dsa_signer(struct signature_algorithm *c,
 {
   CAST(dsa_algorithm, self, c);
   NEW(dsa_signer, res);
-  init_dsa_public(&res->public);
   mpz_init(res->a);
 
 #if 0
@@ -582,13 +579,13 @@ make_dsa_signer(struct signature_algorithm *c,
 #endif
   
   if ( (SEXP_LEFT(i) == 5)
-       && spki_init_dsa_public(&res->public, i)
+       && ( (res->verifier = make_dsa_verifier_internal(i)) )
        && sexp_get_un(i, ATOM_X, res->a, DSA_MAX_QSIZE) )
     {
       res->random = self->random;
       res->super.sign = do_dsa_sign;
       res->super.sign_spki = do_dsa_sign_spki;
-      res->super.public_key = do_dsa_public_key;
+      res->super.get_verifier = do_dsa_get_verifier;
       
       return &res->super;
     }
@@ -599,27 +596,6 @@ make_dsa_signer(struct signature_algorithm *c,
     }
 }
 
-static struct verifier *
-make_dsa_verifier(struct signature_algorithm *self UNUSED,
-		  struct sexp_iterator *i)
-{
-  NEW(dsa_verifier, res);
-  init_dsa_public(&res->public);
-
-  if ( (SEXP_LEFT(i) == 4)
-       && spki_init_dsa_public(&res->public, i))
-    {
-      res->super.verify = do_dsa_verify;
-      res->super.verify_spki = do_dsa_verify_spki;
-      return &res->super;
-    }
-  else
-    {
-      KILL(res);
-      return NULL;
-    }
-}
-  
 struct signature_algorithm *
 make_dsa_algorithm(struct randomness *random)
 {
@@ -633,62 +609,17 @@ make_dsa_algorithm(struct randomness *random)
 }
 
 
-/* Constructor using a key of type ssh-dss */
-int parse_dsa_public(struct simple_buffer *buffer,
-		     struct dsa_public *public)
-{
-  return (parse_bignum(buffer, public->p, DSA_MAX_SIZE)
-	  && (mpz_sgn(public->p) == 1)
-	  && parse_bignum(buffer, public->q, DSA_MAX_QSIZE)
-	  && (mpz_sgn(public->q) == 1)
-	  && (mpz_cmp(public->q, public->p) < 0) /* q < p */ 
-	  && parse_bignum(buffer, public->g, DSA_MAX_SIZE)
-	  && (mpz_sgn(public->g) == 1)
-	  && (mpz_cmp(public->g, public->p) < 0) /* g < p */ 
-	  && parse_bignum(buffer, public->y, DSA_MAX_SIZE) 
-	  && (mpz_sgn(public->y) == 1)
-	  && (mpz_cmp(public->y, public->p) < 0) /* y < p */  );
-}
-
-struct dsa_verifier *
+struct verifier *
 make_ssh_dss_verifier(UINT32 public_length,
 		      const UINT8 *public)
 {
-  NEW(dsa_verifier, res);
   struct simple_buffer buffer;
   int atom;
-
-  /* FIXME: The allocator could do this kind of initialization
-   * automatically. */
-  init_dsa_public(&res->public);
   
   simple_buffer_init(&buffer, public_length, public);
-  if (!parse_atom(&buffer, &atom)
-      || (atom != ATOM_SSH_DSS) )
-    {
-      KILL(res);
-      return 0;
-    }
-  
-  if (!parse_dsa_public(&buffer, &res->public))
-    {
-      /* FIXME: Perhaps do some more sanity checks? */
-      KILL(res);
-      return NULL;
-    }
 
-  res->super.verify = do_dsa_verify;
-  res->super.verify_spki = do_dsa_verify_spki;
-  
-  return res;
-}
-
-struct lsh_string *
-ssh_dss_public_key(struct signer *s)
-{
-  CAST(dsa_signer, dsa, s);
-  return ssh_format("%a%n%n%n%n",
-		    ATOM_SSH_DSS,
-		    dsa->public.p, dsa->public.q,
-		    dsa->public.g, dsa->public.y);
+  return ( (parse_atom(&buffer, &atom)
+	    && (atom == ATOM_SSH_DSS))
+	   ? parse_ssh_dss_public(&buffer)
+	   : NULL);
 }
