@@ -1,6 +1,6 @@
 /* keyexchange.c
  *
- * $Id: keyexchange.c,v 1.72 2000/10/01 19:07:32 nisse Exp $ */
+ * $Id: keyexchange.c,v 1.73 2000/11/14 17:52:35 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -730,7 +730,7 @@ do_kexinit_filter(struct command *s,
   COMMAND_RETURN(c, self->init);
 }
 
-COMMAND_SIMPLE(kexinit_filter)
+DEFINE_COMMAND_SIMPLE(kexinit_filter, a)
 {
   CAST(simple_kexinit, init, a);
   NEW(kexinit_filter_command, self);
