@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.128 2000/11/26 23:25:55 nisse Exp $ */
+ * $Id: lsh.c,v 1.129 2000/12/11 21:40:31 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -269,7 +269,8 @@ do_options2identities(struct command *ignored UNUSED,
       COMMAND_RETURN(c, make_object_list(0, -1));
     }
   else
-    COMMAND_CALL(make_spki_read_userkeys(options->signature_algorithms,
+    COMMAND_CALL(make_spki_read_userkeys(options->algorithms->algorithms,
+					 options->signature_algorithms,
 					 options->super.tty),
 		 f, c, e);
   
