@@ -2,7 +2,7 @@
  *
  * user authorization database
  *
- * $Id: server_authorization.h,v 1.3 1999/11/16 22:13:40 nisse Exp $ */
+ * $Id: server_authorization.h,v 1.4 1999/11/28 18:34:26 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -31,7 +31,7 @@
 
 struct lookup_verifier *
 make_authorization_db(struct lsh_string *index_name, 
-		      struct signature_algorithm *s,
+		      /* struct signature_algorithm *s, */
 		      struct hash_algorithm *h);
 
 #endif /* LSH_SERVER_AUTHORIZATION_H_INCLUDED */
