@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.52 2000/11/15 17:31:40 nisse Exp $
+ * $Id: server_session.c,v 1.56 2000/11/26 23:25:59 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -75,8 +75,9 @@
        ; Child process's stdio 
        (in object lsh_fd)
        (out object lsh_fd)
-       ;; err may be NULL, if there's no separate stderr channel.
-       ;; This happens if we use a pty, and the bash workaround
+
+       ; err may be NULL, if there's no separate stderr channel.
+       ; This happens if we use a pty, and the bash workaround is used.
        (err object lsh_fd)))
 */
 
@@ -183,9 +184,7 @@ make_server_session(UINT32 initial_window,
 static void
 do_open_session(struct channel_open *s,
 		struct ssh_connection *connection UNUSED,
-		UINT32 type UNUSED,
-		UINT32 send_window_size UNUSED,
-		UINT32 send_max_packet UNUSED,
+		struct channel_open_info *info UNUSED,
 		struct simple_buffer *args,
 		struct command_continuation *c,
 		struct exception_handler *e)
@@ -306,44 +305,6 @@ make_exit_shell(struct server_session *session)
        (reaper object reap)))
 */
 
-#if 0
-/* Creates a one-way socket connection. Returns 1 on success, 0 on
- * failure. fds[0] is for reading, fds[1] for writing (like for the
- * pipe() system call). */
-static int
-make_pipe(int *fds)
-{
-  if (socketpair(AF_UNIX, SOCK_STREAM, 0, fds) < 0)
-    {
-      werror("socketpair() failed: %z\n", STRERROR(errno));
-      return 0;
-    }
-  debug("Created socket pair. Using fd:s %i <-- %i\n", fds[0], fds[1]);
-
-  if (SHUTDOWN(fds[0], SHUT_WR) < 0)
-    {
-      werror("shutdown(%i, SEND) failed: %z\n", fds[0], STRERROR(errno));
-      goto fail;
-    }
-  if (SHUTDOWN(fds[1], SHUT_RD) < 0)
-    {
-      werror("shutdown(%i, REC) failed: %z\n", fds[0], STRERROR(errno));
-    fail:
-      {
-	int saved_errno = errno;
-
-	close(fds[0]);
-	close(fds[1]);
-
-	errno = saved_errno;
-	return 0;
-      }
-    }
-  
-  return 1;
-}
-#endif
-
 static int
 make_pipes(int *in, int *out, int *err)
 {
@@ -642,8 +603,7 @@ static void
 do_spawn_shell(struct channel_request *c,
 	       struct ssh_channel *channel,
 	       struct ssh_connection *connection,
-	       UINT32 type UNUSED,
-	       int want_reply UNUSED,
+	       struct channel_request_info *info UNUSED,
 	       struct simple_buffer *args,
 	       struct command_continuation *s,
 	       struct exception_handler *e)
@@ -739,8 +699,7 @@ static void
 do_spawn_exec(struct channel_request *c,
 	      struct ssh_channel *channel,
 	      struct ssh_connection *connection,
-	      UINT32 type UNUSED,
-	      int want_reply UNUSED,
+	      struct channel_request_info *info UNUSED,
 	      struct simple_buffer *args,
 	      struct command_continuation *s,
 	      struct exception_handler *e)
@@ -850,8 +809,7 @@ static void
 do_alloc_pty(struct channel_request *c UNUSED,
 	     struct ssh_channel *channel,
 	     struct ssh_connection *connection,
-	     UINT32 type UNUSED,
-	     int want_reply UNUSED,
+	     struct channel_request_info *info UNUSED,
 	     struct simple_buffer *args,
 	     struct command_continuation *s,
 	     struct exception_handler *e)
